//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//Some code from Amuc.cpp by Herbert

#pragma once
#include <deque>
#include "Ping.h"
#include "Opcodes.h"
#include "Log.h"

#define MAX_AVE_TIMES MAXAVERAGETIME/100+1
#define STATS_AVE_TIMES MAXAVERAGETIME/100
#define AVE_TIMES 50

typedef struct
{
    DWORD Bps;          //bytes per second          
    DWORD ave;          //average throughput
    DWORD time;         //time in milliseconds that the sample was taken
    uint64 total;       //total received or sent in bits
} STATS_STRUCT;


class CBandwidthControl : public CObject
{

DECLARE_DYNCREATE(CBandwidthControl)

public:
	CBandwidthControl();
	~CBandwidthControl(void);
	float	GetMaxUpload()							{return retUpload;}
	float	GetCalcUpload()							{return upload;}
	float	GetMaxDownload();
	bool	IsSessionLimit()						{return sessionLimit;}
	float   GetMaxDownloadOnUpload(float upload, float maxdownload, float isConnectionOH);
	float	GetCurrDataUpload()						{return eMuleUpDataRate;}
	float	GetCurrUpload()							{return eMuleUpRate;}
	float	GetCurrDataDownload()					{return eMuleDownDataRate;}
	float	GetCurrDownload()						{return eMuleDownRate;}
	bool	IsMinUpReached()						{return	minUpReached;}
	float	GetCurrStatsUpload()					{return eMuleStatsUpRate;}
	float	GetCurrStatsDataUpload()				{return eMuleStatsUpDataRate;}
	float	GetCurrStatsDownload()					{return eMuleStatsDownRate;}
	float	GetCurrStatsDataDownload()				{return eMuleStatsDownDataRate;}
	float	GetCurrPing()							{return avg_ping;}
	float	GetBasePingUp()							{return base_ping_up;}
	uint64	GetTotalSentBytes()						{return m_dbTotalUp;}
	uint64	GetTotalReceivedBytes()					{return m_dbTotalDown;}
	float	GetCurrOverhead()						{return eMuleoverhead;}
	float	GetMaxDataRate()						{return maxUpDataRate;}

	bool	SetIP(CString IP);
	bool	WasInitialized()						{return wasInit;}
	bool	IsPingWorking()							{return (pingThread ? pingThread->isWorking() : false);}
	bool	IsPingPreparing()						{return (pingThread ? pingThread->isPreparing() : false);}
	CString	GetPingedServer()						{return (pingThread ? pingThread->GetPingedServer() : _T("-"));}
	// ZZ:UploadSpeedSense -->
	bool	AddHostsToCheck(CTypedPtrList<CPtrList, CServer*> &list)	{if (pingThread) return pingThread->AddHostsToCheck(list); else return false;}
    bool	AddHostsToCheck(CTypedPtrList<CPtrList, CUpDownClient*> &list)	{if (pingThread) return pingThread->AddHostsToCheck(list); else return false;}
	// ZZ:UploadSpeedSense <--
	// [ionix] - PS
	void	SetPSSessionLimitCounter(bool add)		{if(add) m_uPSSessionLimitCounter+=1; else if(m_uPSSessionLimitCounter<=0) LogError(_T("The PowershareLimitCounter must not be negative. Please report this error to the iONiX devs")); else m_uPSSessionLimitCounter-=1;}
	bool	IsPSSessionLimit()						{return m_uPSSessionLimitCounter>0;}
	// [ionix] - PS
	void	AddUp(uint32 up, bool fileData=false);
	void	AddDown(uint32 down, uint32 oh, bool fileData=false);
	void	StartBandwidthControl();
	void	EndBandwidthControl(void);
	virtual int Process(); 
	virtual	BOOL	InitInstance()		{return true;}

private:
	bool Init();
	void StartPinging();
	void StopPinging();
	void ReCalculateBasePing(float lowestPing);
	void CalcAverages( uint64 dwTotal, DWORD dwTime, DWORD dwBPS, STATS_STRUCT* pStats, int g_nAveraging=AVE_TIMES ); //from AMUC - Herbert
	
	CPingThread*	pingThread;
	
	STATS_STRUCT eMuleUpAvg[MAX_AVE_TIMES];
	STATS_STRUCT eMuleUpDataAvg[MAX_AVE_TIMES];
	STATS_STRUCT eMuleUpDataStats[MAX_AVE_TIMES];
	STATS_STRUCT eMuleUpStats[MAX_AVE_TIMES];
	STATS_STRUCT eMuleDownAvg[MAX_AVE_TIMES];
	STATS_STRUCT eMuleDownStats[MAX_AVE_TIMES];
	STATS_STRUCT eMuleDownDataAvg[MAX_AVE_TIMES];
	STATS_STRUCT eMuleDownDataStats[MAX_AVE_TIMES];

	float		upload;			// upload calculated next
	float		uploadTemp;
	float		uploadACK;
	bool		useUploadTemp;
	float		download;			// download calculated next
	float		downloadTemp;
	float		downloadACK;
	bool		useDownloadTemp;
	float		eMuleUpRateTemp;
	float		eMuleDownRateTemp;
	float		eMuleStatsUpRateTemp;
	float		eMuleStatsDownRateTemp;
	DWORD		dwTime;
	uint64		dbUp, dbUpData, dbDown, dbDownData;

	std::deque<float> bandwidth;
	float bandwidthAvg;
	float currUpload;
	float retUpload;
	bool  minUpReached;
	float currDownload;
	float retDownload;
	float maxUpDataRate;
	float maxDownDataRate;
	uint64 eMuleUp;
	uint64 eMuleUpData;
	uint64 eMuleDown;
	uint64 eMuleDownData;
	float avg_ping;
	float base_ping_up;
	float base_ping_down;

	int m_nArrayIndex;
    uint64 m_dbTotalUp;
	uint64 m_dbTotalUpData;
    uint64 m_dbTotalDown;
    uint64 m_dbTotalDownData;
	
	float		eMuleUpDataRate;		// Datarate of the current eMule upload
	float		eMuleUpRate;
	float		eMuleStatsUpRate;
	float		eMuleStatsUpDataRate;
	float		eMuleDownRate;
	float		eMuleDownDataRate;
	float		eMuleStatsDownRate;
	float		eMuleStatsDownDataRate;
	bool		sessionLimit;

	float		otherUpload;	// Datarate of the current upload of other applications
	float		otherDown;
	float		eMuleoverhead;

	bool		pingThreadFailed;
	bool		wasInit;
	uint8		pingUpSliderPos;
	uint8		pingDownSliderPos;
	uint16		m_uPSSessionLimitCounter; // [ionix] - PS
};
